#!/usr/bin/perl

use strict;

# .model D D
# .lib C:\users\toni\Meine Dokumente\LTspiceXVII\lib\cmp\standard.dio
# .model NPN NPN
# .model PNP PNP
# .lib C:\users\toni\Meine Dokumente\LTspiceXVII\lib\cmp\standard.bjt
# .model NMOS NMOS
# .model PMOS PMOS
# .lib C:\users\toni\Meine Dokumente\LTspiceXVII\lib\cmp\standard.mos

my %fix_lines=(
	"\b5"				=>	'u',	# µ in iso
	'µ'				=>	'u',
	'\s*;.*'			=>	'',			# remove comment lines
	'.backanno'			=>	'',
	'.op.*'				=>	'',
	'.model D D'			=>	'',
	'.model NPN NPN'		=>	'',
	'.model PNP PNP'		=>	'',
	'.model NMOS NMOS'		=>	'',
	'.model PMOS PMOS'		=>	'',
	'.lib .*standard.bead'		=>	'.lib standard.bead',
	'.lib .*standard.bjt'		=>	'.lib standard.bjt',
	'.lib .*standard.cap'		=>	'.lib standard.cap',
	'.lib .*standard.dio'		=>	'.lib standard.dio',
	'.lib .*standard.ind'		=>	'.lib standard.ind',
	'.lib .*standard.jft'		=>	'.lib standard.jft',
	'.lib .*standard.mos'		=>	'.lib standard.mos',
	'.lib .*standard.res'		=>	'.lib standard.res',
);

my %fix_chars=(
	"\xb5"				=>	'u',	# µ in iso
	'µ'				=>	'u',
);

my @ignlines;

my $add_currents=1;

open(NETLIST, "< $ARGV[0]") && do {
	while (my $line = <NETLIST>) {
		chomp($line);
		my $found_match=0;
		# replace special chars like micro
		foreach my $c (keys %fix_chars) {
			my $regex=$c;
			my $replace=$fix_chars{$c};
			my $oline=$line;
			$line=~s/$regex/$replace/g;
			if ($line ne $line) {
				print STDERR "REPLACE string '$c' by '$replace' in <$line>\n";
			}
		}

		FLOOP: foreach my $f (keys %fix_lines) {
			my $regex='^' . $f . '$';
			if ($line=~/$regex/i) {
				if ($fix_lines{$f} ne '') {
					print STDERR "REPLACE line <$line> by <$fix_lines{$f}>\n";
					print "$fix_lines{$f}\n";
				} else {
					print STDERR "REMOVE line <$line>\n";
				}
				$found_match=1;
				next FLOOP;
			}
		}
		# and some special cases
		if (! $found_match) {
			# fix param has no '="
			if ($line=~/^.param\s+([^\s=]+)\s+([^\s=]+)$/i) {
				print STDERR "PARAM: NO '=': $line\n";
				print ".param $1 = $2\n";

			} elsif ($line=~/^([rlc][^\s]+)\s+([^\s]+)\s+([^\s]+)\s+(\{.*\}|[^\s]+)/i) {
				# only first 4 parameters
				print STDERR "R/C remove 5 - nth parameter: $line\n";
				print "$1 $2 $3 $4\n";

			} elsif ($line=~/^([m][^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+(.*)/i) {
				print STDERR "MOS remove 5 parameter: $line\n";
				print "$1 $2 $3 $4 $6\n";
				#	Error on line 121 :
				#	  m1 +v n010 vem vem ixth80n20l
				#	too many nodes connected to instance
				#	Error on line 122 :
				#	  m2 -v n045 vnem vnem ixth48p20p

			} elsif ($add_currents && $line=~/^.end/i) {
				$add_currents=0;
				print ".option savecurrents\n.end\n";
			} else {
				print "$line\n";
			}
		}
	}
}
